function [data,cparams]= datamaker(Efr,Kd,n,A0mu,D0mu,r,sig,plot_on)
% [data,cparams]= datamaker(Efr,Kd,n,A0mu,D0mu,r,plot_on)
% This function simulates single-fluorophore calibration data 
% and three-cube FRET data 
% for multiple cells/samples using matrix model:  
%    [a11 a12 a13] [D ] [Id]
%    [a21 a22 a23]*[A ]=[Ia]
%    [a31 a32 a33] [AD] [If]
%%% INPUT (Efr,Kd,n,A0mu,D0mu,r,plot_on)
    % Efr: FRET efficiency value, should be between 0 and 1.
    % Kd: Dissociation constant e.g. 1e-6 
    % n: number of measurement repeats for each sample
    % A0mu,D0mu: vectors of equal length m, concentrations in each sample 
    % r: scalar noise level (normal or lognormal) ex: 0.1= 10% of data mean
    % sig: adds noise to photophysical params (a=a+sig*randn).
    % plot_on: 1 to plot, 0 to not plot
%%% OUTPUT [data,Efr,Kd,kd_eff,cparams,testd,testa,testad]
    % data: structure of length m, containing fields:
    %     .ydata: n*3 matrix of data [Id Ia If]
    %     .noisetype: 'normal' or 'lognormal'
    %     .sigdat: standard dev of data
    %     .sd1,sa1,sf1: sum of donor, acceptor, fret data
    %     .sd2,sa2,sf2: sum of donor, acceptor, fret data squared
    % cparams: matrix of photophysical constants used
%
% C. Lichten, University of Edinburgh, 2010
% http://swainlab.bio.ed.ac.uk/software.php

addpath('utils/');
addpath('utils/mcmcstat/');

% default values:
if nargin==0
   Efr= 0.4; Kd= 1e-6; n= 10; 
   A0mu= 1e-6*[.2 1 5]; D0mu= A0mu;
   r= .05; sig= 0; plot_on= 1;
end
    

a= allvalues_subfun; 
% add noise to a (the photophysical constants)
% the a~=0 means only nonzero params get noise, 0 stays 0.  
cparams= a;
a= a+sig*(a~=0)*randn(size(a));
cparams_used=a;
a11= a(1,1); a12= a(1,2); a13= a(1,3);
a21= a(2,1); a22= a(2,2); a23= a(2,3);
a31= a(3,1); a32= a(3,2); a33= a(3,3);

K= 1/Kd;

%% Brightness vs. Concentration calibration (not currently in output)
ra= 0; rd= 0; % noise in calibrations
% ACCEPTOR
aconc_ideal= logspace(-1,1,30);
aconc_real = aconc_ideal; %+1e-1*randn(size(aconc_ideal)));
%(in uM) includes measurement error. could add pipetting (x) error eventually
Aobs_D= exp(log(aconc_real*a12)+ra*randn(size(aconc_ideal)));
Aobs_A= exp(log(aconc_real*a22)+ra*randn(size(aconc_ideal)));
Aobs_F= exp(log(aconc_real*a32)+ra*randn(size(aconc_ideal)));

% DONOR
dconc_ideal= logspace(-1,1,30);
dconc_real = dconc_ideal; %+1e-1*randn(size(dconc_ideal)));
%(in uM) includes measurement error. could add pipetting (x) error eventually
Dobs_D= exp(log(dconc_real*a11)+rd*randn(size(dconc_ideal)));
Dobs_A= exp(log(dconc_real*a21)+rd*randn(size(dconc_ideal)));
Dobs_F= exp(log(dconc_real*a31)+rd*randn(size(dconc_ideal)));

% if plot_on==1
%     %Plotting
%     % plot brightness vs. conc. for calibration
%     figure(201), plot(aconc_ideal,Aobs_D,'ro'), hold on
%     plot(aconc_ideal,Aobs_A,'rx'); plot(aconc_ideal,Aobs_F,'rv')
%     plot(dconc_ideal,Dobs_D,'bo'), hold on
%     plot(dconc_ideal,Dobs_A,'bx'); plot(dconc_ideal,Dobs_F,'bv')
%     title('brightness vs. concentration for Calibration')
%     xlabel('Concentration (M)'), ylabel('Fluorescence (A.U.)')
%     legend('Acc: donor filter','Acc: acc filter','Acc: fret filter',...
%         'Don: donor filter','Don: acc filter','Don: fret filter');
% end

%% EXPERIMENTAL PART
for i= 1:length(A0mu)
    %% ith cell
    A0i= A0mu(i)*ones(n,1);
    D0i= D0mu(i);
    
    q= sqrt((A0i-D0i).^2*K.^2 + 1 + 2*K*(A0i + D0i))./(2*K);
    % second one is slightly simpler
    AD1= max((1+A0i*K+D0i*K)/(2*K)-q,0);
    D1= D0i-AD1;
    A1= A0i-AD1;
    
    if abs(A1(1)*D1(1)/AD1(1)-Kd)>(Kd/1e10) % if you don't get a good Kd
        % to get concentration, it's often more accurate this way, to
        % avoid potential numerical errors
        options= optimset('Display','off','TolFun',min([D0mu A0mu])*1e-5);
        func= @(x) (A0mu(i)-exp(x)).*(D0mu(i)-exp(x))./exp(x)-1./K;
        x0= 2*log(min([D0mu(i) A0mu(i)])/2);  %[D0i/2; A0i/2; min([D0mu(i) A0mu(i)])/2];
        [y,out]= fsolve(func,x0,options);
        AD= exp(y); D= D0i-AD;  A= A0i-AD;
        if A(1)<0 || AD(1)<0 || D(1)<0
            A= A1; D= D1; AD= AD1;
        end
    else
        A= A1; D= D1; AD= AD1;
    end
    
    testd(i)= D(1);    testa(i)= A(1);    testad(i)= AD(1);
    kd_eff(i)= testd(i)*testa(i)./testad(i);
    
    Dpred= a11*D + a12*A + (a11*(1-Efr)+a12+a13*Efr)*AD;
    Apred= a21*D + a22*A + (a21*(1-Efr)+a22+a23*Efr)*AD;
    Fpred= a31*D + a32*A + (a31*(1-Efr)+a32+a33*Efr)*AD;
    data1= [Dpred Apred Fpred];

    %lognormal measurement noise
%     data1(:,1)= exp(log(data1(:,1)) + r*randn(size(data1,1),1));
%     data1(:,2)= exp(log(data1(:,2)) + r*randn(size(data1,1),1));
%     data1(:,3)= exp(log(data1(:,3)) + r*randn(size(data1,1),1));
%     type= 'lognormal';
    
    %normal measurement noise
    data1(:,1)= data1(:,1) + r*mean(data1(:,1))*randn(size(data1,1),1);
    data1(:,2)= data1(:,2) + r*mean(data1(:,2))*randn(size(data1,1),1);
    data1(:,3)= data1(:,3) + r*mean(data1(:,3))*randn(size(data1,1),1);
    type= 'normal';
    
    data(i).ydata=  data1;    data(i).A0i= A0i;    data(i).D0i= D0i;
    data(i).A0mu= A0mu(i);    data(i).D0mu= D0mu(i); data(i).sig=sig;
    data(i).kd_eff=kd_eff(i); data(i).noisetype=type; data(i).noise=r;
    data(i).cparams= cparams; data(i).cparams_used = cparams_used;  
    data(i).Efr= Efr; data(i).Kd= Kd;
end

%% Plotting
if plot_on==1
    DataPlotter
end

for i= 1:length(data)
    data(i).sigdat= std(data(i).ydata);
    data(i).sd1= sum(data(i).ydata(:,1));
    data(i).sa1= sum(data(i).ydata(:,2));
    data(i).sf1= sum(data(i).ydata(:,3));
    data(i).sd2= sum(data(i).ydata(:,1).^2);
    data(i).sa2= sum(data(i).ydata(:,2).^2);
    data(i).sf2= sum(data(i).ydata(:,3).^2);    
end 

function a= allvalues_subfun
% subfunction contains values for photophysical constants (aij) where the
% form is: [a11 a12 a13] [D ] [Id]
%          [a21 a22 a23]*[A ]=[Ia]
%          [a31 a32 a33] [AD] [If]

% a= [3.1e2 1  0; 0 7.5e2 0; 2.4e1 5e1 3.1e2]; 
% a= 1e-6*[1.584e9 0 0;5.626e6 5.11e7 2.7e7; 1.505e8 4.32e7 9.6e8];
%  a= [.6 0 0 ; 0 1.3 0; .4 .6 1.8];
% a= [.8 .02 .4*.02; .1 2.6 .1*1;.3 .4 .6*.4];  
a= [.8 .02 .4*.02; .1 1 .1*1;.17 .2 .6*.4];  
% a= [1.2 0 0 ; 0 1.3 0; 0 0 .8]; %no x-talk
% a= 1e-6*[1.584e9 0 0;5.626e6 1/3*1.69e9 8.53e5; 1.505e8 1/3*1.426e8 1.37e9];
% a= [164.4 .04  0; 0 80 0; 105.2 80 274.09]; 


